/*
    Парус 8 - Панели мониторинга - Редактор панелей
    Компоненты: График (представление)
*/

//---------------------
//Подключение библиотек
//---------------------

import React from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Paper } from "@mui/material"; //Интерфейсные элементы
import { P8PChart } from "../../../../components/p8p_chart"; //График
import { useComponentDataSource } from "../components_hooks"; //Хуки для данных
import { DATA_SOURCE_SHAPE } from "../editors_common"; //Общие объекты компонентов
import { COMPONENT_MESSAGE_TYPE, COMPONENT_MESSAGES, ComponentInlineMessage } from "../views_common"; //Общие компоненты представлений
import "../../panels_editor.css"; //Стили редактора

//---------
//Константы
//---------

//Иконка компонента
const COMPONENT_ICON = "bar_chart";

//Наименование компонента
const COMPONENT_NAME = "График";

//Стили
const STYLES = {
    CHART: { width: "100%", height: "100%", alignItems: "center", justifyContent: "center", display: "flex" }
};

//-----------
//Тело модуля
//-----------

//График (представление)
const Chart = ({ dataSource = null, values = {} } = {}) => {
    //Собственное состояние - данные
    const [data, error] = useComponentDataSource({ dataSource, values });

    //Флаг настроенности графика
    const haveConfing = dataSource?.stored ? true : false;

    //Флаг наличия данных
    const haveData = data?.init === true && !error ? true : false;

    //Данные графика
    const chart = data?.XCHART || {};

    //Формирование представления
    return (
        <Paper className={"component-view__container component-view__container__empty"} elevation={6}>
            {haveConfing && haveData ? (
                <P8PChart style={STYLES.CHART} {...chart} />
            ) : (
                <ComponentInlineMessage
                    icon={COMPONENT_ICON}
                    name={COMPONENT_NAME}
                    message={!haveConfing ? COMPONENT_MESSAGES.NO_SETTINGS : error ? error : COMPONENT_MESSAGES.NO_DATA_FOUND}
                    type={error ? COMPONENT_MESSAGE_TYPE.ERROR : COMPONENT_MESSAGE_TYPE.COMMON}
                />
            )}
        </Paper>
    );
};

//Контроль свойств компонента - График (представление)
Chart.propTypes = {
    dataSource: DATA_SOURCE_SHAPE,
    values: PropTypes.object
};

//----------------
//Интерфейс модуля
//----------------

export default Chart;
